@@ -18,8 +18,12 @@ from calibre.utils.localization import get_lang
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.imghdr import what
 
+
 def sanitize_file_name(x):
-    return re.sub(r'[?&=;]', '_', ascii_filename(x))
+    ans = re.sub(r'\s+', ' ', re.sub(r'[?&=;#]', '_', ascii_filename(x))).strip().rstrip('.')
+    ans, ext = ans.rpartition('.')[::2]
+    return (ans.strip() + '.' + ext.strip()).rstrip('.')
+
 
 class HTMLInput(InputFormatPlugin):
 
@@ -27,6 +31,7 @@ class HTMLInput(InputFormatPlugin):
     author      = 'Kovid Goyal'
     description = 'Convert HTML and OPF files to an OEB'
     file_types  = set(['opf', 'html', 'htm', 'xhtml', 'xhtm', 'shtm', 'shtml'])
+    commit_name = 'html_input'
 
     options = set([
         OptionRecommendation(name='breadth_first',
@@ -89,8 +94,7 @@ class HTMLInput(InputFormatPlugin):
             return self._is_case_sensitive
         if not path or not os.path.exists(path):
             return islinux or isbsd
-        self._is_case_sensitive = not (os.path.exists(path.lower()) and
-                                       os.path.exists(path.upper()))
+        self._is_case_sensitive = not (os.path.exists(path.lower()) and os.path.exists(path.upper()))
         return self._is_case_sensitive
 
     def create_oebbook(self, htmlpath, basedir, opts, log, mi):
@@ -310,4 +314,3 @@ class HTMLInput(InputFormatPlugin):
             self.log.exception('Failed to read CSS file: %r'%link)
             return (None, None)
         return (None, raw)
-
