@@ -24,7 +24,7 @@ XML_ENTITIES = {'lt', 'gt', 'amp', 'apos', 'quot'}
 ALL_ENTITIES = HTML_ENTITTIES | XML_ENTITIES
 
 replace_pat = re.compile('&(%s);' % '|'.join(re.escape(x) for x in sorted((HTML_ENTITTIES - XML_ENTITIES))))
-mismatch_pat = re.compile('tag mismatch:.+?line (\d+).+?line \d+')
+mismatch_pat = re.compile(r'tag mismatch:.+?line (\d+).+?line \d+')
 
 
 class EmptyFile(BaseError):
@@ -59,7 +59,7 @@ class XMLParseError(BaseError):
     is_parsing_error = True
 
     HELP = _('A parsing error in an XML file means that the XML syntax in the file is incorrect.'
-             ' Such a file will most probably not open in an ebook reader. These errors can '
+             ' Such a file will most probably not open in an e-book reader. These errors can '
              ' usually be fixed automatically, however, automatic fixing can sometimes '
              ' "do the wrong thing".')
 
@@ -80,6 +80,13 @@ class HTMLParseError(XMLParseError):
              ' however, automatic fixing can sometimes "do the wrong thing".')
 
 
+class PrivateEntities(XMLParseError):
+
+    HELP = _('This HTML file uses private entities.'
+    ' These are not supported. You can try running "Fix HTML" from the Tools menu,'
+    ' which will try to automatically resolve the private entities.')
+
+
 class NamedEntities(BaseError):
 
     level = WARN
@@ -106,21 +113,26 @@ class NamedEntities(BaseError):
         return changed
 
 
+def make_filename_safe(name):
+    from calibre.utils.filenames import ascii_filename
+
+    def esc(n):
+        return ''.join(x if x in URL_SAFE else '_' for x in n)
+    return '/'.join(esc(ascii_filename(x)) for x in name.split('/'))
+
+
 class EscapedName(BaseError):
 
     level = WARN
 
     def __init__(self, name):
-        from calibre.utils.filenames import ascii_filename
         BaseError.__init__(self, _('Filename contains unsafe characters'), name)
         qname = urlquote(name)
 
-        def esc(n):
-            return ''.join(x if x in URL_SAFE else '_' for x in n)
-        self.sname = '/'.join(esc(ascii_filename(x)) for x in name.split('/'))
+        self.sname = make_filename_safe(name)
         self.HELP = _(
             'The filename {0} contains unsafe characters, that must be escaped, like'
-            ' this {1}. This can cause problems with some ebook readers. To be'
+            ' this {1}. This can cause problems with some e-book readers. To be'
             ' absolutely safe, use only the English alphabet [a-z], the numbers [0-9],'
             ' underscores and hyphens in your file names. While many other characters'
             ' are allowed, they may cause problems with some software.').format(name, qname)
@@ -144,7 +156,7 @@ class TooLarge(BaseError):
     level = INFO
     MAX_SIZE = 260 *1024
     HELP = _('This HTML file is larger than %s. Too large HTML files can cause performance problems'
-             ' on some ebook readers. Consider splitting this file into smaller sections.') % human_readable(MAX_SIZE)
+             ' on some e-book readers. Consider splitting this file into smaller sections.') % human_readable(MAX_SIZE)
 
     def __init__(self, name):
         BaseError.__init__(self, _('File too large'), name)
@@ -238,6 +250,7 @@ def check_html_size(name, mt, raw):
         errors.append(TooLarge(name))
     return errors
 
+
 entity_pat = re.compile(br'&(#{0,1}[a-zA-Z0-9]{1,8});')
 
 
@@ -249,9 +262,16 @@ def check_encoding_declarations(name, container):
     return errors
 
 
+def check_for_private_entities(name, raw):
+    if re.search(br'<!DOCTYPE\s+.+?<!ENTITY\s+.+?]>', raw, flags=re.DOTALL) is not None:
+        return True
+
+
 def check_xml_parsing(name, mt, raw):
     if not raw:
         return [EmptyFile(name)]
+    if check_for_private_entities(name, raw):
+        return [PrivateEntities(_('Private entities found'), name)]
     raw = raw.replace(b'\r\n', b'\n').replace(b'\r', b'\n')
     # Get rid of entities as named entities trip up the XML parser
     eproc = EntitityProcessor(mt)
@@ -320,6 +340,7 @@ class CSSError(BaseError):
                     elem.set('style', force_unicode(container.parse_css(raw, is_declaration=True).cssText, 'utf-8').replace('\n', ' '))
         return True
 
+
 pos_pats = (re.compile(r'\[(\d+):(\d+)'), re.compile(r'(\d+), (\d+)\)'))
 
 
@@ -467,6 +488,7 @@ def check_filenames(container):
             errors.append(EscapedName(name))
     return errors
 
+
 valid_id = re.compile(r'^[a-zA-Z][a-zA-Z0-9_:.-]*$')
 
 
