@@ -15,9 +15,11 @@ from calibre.ebooks.oeb.polish.errors import MalformedMarkup
 from calibre.ebooks.oeb.polish.toc import node_from_loc
 from calibre.ebooks.oeb.polish.replace import LinkRebaser
 
+
 class AbortError(ValueError):
     pass
 
+
 def in_table(node):
     while node is not None:
         if node.tag.endswith('}table'):
@@ -25,6 +27,7 @@ def in_table(node):
         node = node.getparent()
     return False
 
+
 def adjust_split_point(split_point, log):
     '''
     Move the split point up its ancestor chain if it has no content
@@ -49,9 +52,11 @@ def adjust_split_point(split_point, log):
 
     return sp
 
+
 def get_body(root):
     return root.find('h:body', namespaces=XPNSMAP)
 
+
 def do_split(split_point, log, before=True):
     '''
     Split tree into a *before* and an *after* tree at ``split_point``.
@@ -143,6 +148,7 @@ def do_split(split_point, log, before=True):
 
     return tree, tree2
 
+
 class SplitLinkReplacer(object):
 
     def __init__(self, base, bottom_anchors, top_name, bottom_name, container):
@@ -163,6 +169,7 @@ class SplitLinkReplacer(object):
             self.replaced = True
         return url
 
+
 def split(container, name, loc_or_xpath, before=True, totals=None):
     '''
     Split the file specified by name at the position specified by loc_or_xpath.
@@ -213,7 +220,7 @@ def split(container, name, loc_or_xpath, before=True, totals=None):
     bottom_name = container.href_to_name(manifest_item.get('href'), container.opf_name)
 
     # Fix links in the split trees
-    for r, rname, anchors in [(root1, bottom_name, anchors_in_bottom), (root2, name, anchors_in_top)]:
+    for r in (root1, root2):
         for a in r.xpath('//*[@href]'):
             url = a.get('href')
             if url.startswith('#'):
@@ -222,8 +229,16 @@ def split(container, name, loc_or_xpath, before=True, totals=None):
                 fname = container.href_to_name(url, name)
             if fname == name:
                 purl = urlparse(url)
-                if purl.fragment in anchors:
-                    a.set('href', '%s#%s' % (container.name_to_href(rname, name), purl.fragment))
+                if purl.fragment in anchors_in_top:
+                    if r is root2:
+                        a.set('href', '%s#%s' % (container.name_to_href(name, bottom_name), purl.fragment))
+                    else:
+                        a.set('href', '#' + purl.fragment)
+                elif purl.fragment in anchors_in_bottom:
+                    if r is root1:
+                        a.set('href', '%s#%s' % (container.name_to_href(bottom_name, name), purl.fragment))
+                    else:
+                        a.set('href', '#' + purl.fragment)
 
     # Fix all links in the container that point to anchors in the bottom tree
     for fname, media_type in container.mime_map.iteritems():
@@ -247,9 +262,10 @@ def split(container, name, loc_or_xpath, before=True, totals=None):
     container.dirty(container.opf_name)
     return bottom_name
 
+
 def multisplit(container, name, xpath, before=True):
     '''
-    Split the specified file at multiple locations (all tags that match the specified XPath expression. See also: :func:`split`.
+    Split the specified file at multiple locations (all tags that match the specified XPath expression). See also: :func:`split`.
     Splitting automatically migrates all links and references to the affected
     files.
 
@@ -311,9 +327,11 @@ def add_text(body, text):
     else:
         body.text = (body.text or '') + text
 
+
 def all_anchors(root):
     return set(root.xpath('//*/@id')) | set(root.xpath('//*/@name'))
 
+
 def all_stylesheets(container, name):
     for link in XPath('//h:head/h:link[@href]')(container.parsed(name)):
         name = container.href_to_name(link.get('href'), name)
@@ -321,6 +339,7 @@ def all_stylesheets(container, name):
         if typ == 'text/css':
             yield name
 
+
 def unique_anchor(seen_anchors, current):
     c = 0
     ans = current
@@ -329,6 +348,7 @@ def unique_anchor(seen_anchors, current):
         ans = '%s_%d' % (current, c)
     return ans
 
+
 def remove_name_attributes(root):
     # Remove all name attributes, replacing them with id attributes
     for elem in root.xpath('//*[@id and @name]'):
@@ -336,6 +356,7 @@ def remove_name_attributes(root):
     for elem in root.xpath('//*[@name]'):
         elem.set('id', elem.attrib.pop('name'))
 
+
 def merge_html(container, names, master):
     p = container.parsed
     root = p(master)
@@ -420,6 +441,7 @@ def merge_html(container, names, master):
         repl = MergeLinkReplacer(fname, anchor_map, master, container)
         container.replace_links(fname, repl)
 
+
 def merge_css(container, names, master):
     p = container.parsed
     msheet = p(master)
@@ -486,4 +508,3 @@ def merge(container, category, names, master):
         merge_css(container, names, master)
 
     container.dirty(master)
-
