@@ -38,6 +38,24 @@ class AdaptSQP(SearchQueryParser):
         pass
 
 
+def human_readable_interval(secs):
+    secs = int(secs)
+    days = secs // 86400
+    hours = secs // 3600 % 24
+    minutes = secs // 60 % 60
+    seconds = secs % 60
+    parts = []
+    if days > 0:
+        parts.append('%dd' % days)
+    if hours > 0:
+        parts.append('%dh' % hours)
+    if minutes > 0:
+        parts.append('%dm' % minutes)
+    if secs > 0:
+        parts.append('%ds' % seconds)
+    return ' '.join(parts)
+
+
 class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
 
     job_added = pyqtSignal(int)
@@ -128,7 +146,7 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
                     rtime = job.running_time
                     if rtime is None:
                         return None
-                    return ('%dm %ds'%(int(rtime)//60, int(rtime)%60))
+                    return human_readable_interval(rtime)
                 if col == 4 and job.start_time is not None:
                     return (strftime(u'%H:%M -- %d %b', time.localtime(job.start_time)))
             if role == Qt.DecorationRole and col == 0:
@@ -231,6 +249,9 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
     def row_to_job(self, row):
         return self.jobs[row]
 
+    def rows_to_jobs(self, rows):
+        return [self.jobs[row] for row in rows]
+
     def has_device_jobs(self, queued_also=False):
         for job in self.jobs:
             if isinstance(job, DeviceJob):
@@ -282,8 +303,7 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
         for r in xrange(len(self.jobs)):
             self.dataChanged.emit(self.index(r, 0), self.index(r, 0))
 
-    def kill_job(self, row, view):
-        job = self.jobs[row]
+    def kill_job(self, job, view):
         if isinstance(job, DeviceJob):
             return error_dialog(view, _('Cannot kill job'),
                          _('Cannot kill jobs that communicate with the device')).exec_()
@@ -295,8 +315,7 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
                     _('This job cannot be stopped'), show=True)
         self._kill_job(job)
 
-    def kill_multiple_jobs(self, rows, view):
-        jobs = [self.jobs[row] for row in rows]
+    def kill_multiple_jobs(self, jobs, view):
         devjobs = [j for j in jobs if isinstance(j, DeviceJob)]
         if devjobs:
             error_dialog(view, _('Cannot kill job'),
@@ -470,7 +489,7 @@ class JobsButton(QWidget):  # {{{
         self.pi = ProgressIndicator(self, self.style().pixelMetric(QStyle.PM_ToolBarIconSize))
         self._jobs = QLabel('<b>'+_('Jobs:')+' 0 ')
         self._jobs.mouseReleaseEvent = self.mouseReleaseEvent
-        self.shortcut = 'Shift+Alt+J'
+        self.shortcut = 'Alt+Shift+J'
 
         self.l = l = QHBoxLayout(self)
         l.setSpacing(3)
@@ -481,7 +500,7 @@ class JobsButton(QWidget):  # {{{
         self._jobs.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
         self.setCursor(Qt.PointingHandCursor)
         b = _('Click to see list of jobs')
-        self.setToolTip(b + u' (%s)'%self.shortcut)
+        self.setToolTip(b + u' [%s]'%self.shortcut)
         self.action_toggle = QAction(b, parent)
         parent.addAction(self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
@@ -604,8 +623,7 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         self.search.initialize('jobs_search_history',
                 help_text=_('Search for a job by name'))
         self.search.search.connect(self.find)
-        self.search_button.clicked.connect(lambda :
-                self.find(self.search.current_text))
+        connect_lambda(self.search_button.clicked, self, lambda self: self.find(self.search.current_text))
         self.restore_state()
 
     def restore_state(self):
@@ -649,18 +667,18 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         indices = [self.proxy_model.mapToSource(index) for index in
                 self.jobs_view.selectionModel().selectedRows()]
         indices = [i for i in indices if i.isValid()]
-        rows = [index.row() for index in indices]
-        if not rows:
+        jobs = self.model.rows_to_jobs([index.row() for index in indices])
+        if not jobs:
             return error_dialog(self, _('No job'),
                 _('No job selected'), show=True)
         if question_dialog(self, _('Are you sure?'),
                 ngettext('Do you really want to stop the selected job?',
                     'Do you really want to stop all the selected jobs?',
-                    len(rows))):
-            if len(rows) > 1:
-                self.model.kill_multiple_jobs(rows, self)
+                    len(jobs))):
+            if len(jobs) > 1:
+                self.model.kill_multiple_jobs(jobs, self)
             else:
-                self.model.kill_job(rows[0], self)
+                self.model.kill_job(jobs[0], self)
 
     def kill_all_jobs(self, *args):
         if question_dialog(self, _('Are you sure?'),
