@@ -9,10 +9,10 @@ import functools, re, os, traceback, errno, time
 from collections import defaultdict, namedtuple
 from itertools import groupby
 
-from PyQt5.Qt import (QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage,
+from PyQt5.Qt import (QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage, QFont,
         QModelIndex, QDateTime, QColor, QPixmap, QPainter, QApplication)
 
-from calibre import fit_image
+from calibre import fit_image, force_unicode
 from calibre.gui2 import error_dialog
 from calibre.utils.search_query_parser import ParseException
 from calibre.ebooks.metadata import fmt_sidx, authors_to_string, string_to_authors
@@ -33,10 +33,12 @@ from calibre.library.coloring import color_row_key
 
 Counts = namedtuple('Counts', 'library_total total current')
 
+
 def human_readable(size, precision=1):
     """ Convert a size in bytes into megabytes """
     return ('%.'+str(precision)+'f') % ((size/(1024.*1024.)),)
 
+
 TIME_FMT = '%d %b %Y'
 
 ALIGNMENT_MAP = {'left': Qt.AlignLeft, 'right': Qt.AlignRight, 'center':
@@ -44,12 +46,14 @@ ALIGNMENT_MAP = {'left': Qt.AlignLeft, 'right': Qt.AlignRight, 'center':
 
 _default_image = None
 
+
 def default_image():
     global _default_image
     if _default_image is None:
         _default_image = QImage(I('default_cover.png'))
     return _default_image
 
+
 def group_numbers(numbers):
     for k, g in groupby(enumerate(sorted(numbers)), lambda (i, x):i - x):
         first = None
@@ -58,6 +62,7 @@ def group_numbers(numbers):
                 first = last[1]
         yield first, last[1]
 
+
 class ColumnColor(object):  # {{{
 
     def __init__(self, formatter):
@@ -86,6 +91,7 @@ class ColumnColor(object):  # {{{
             pass
 # }}}
 
+
 class ColumnIcon(object):  # {{{
 
     def __init__(self, formatter, model):
@@ -156,6 +162,7 @@ class ColumnIcon(object):  # {{{
             pass
 # }}}
 
+
 class BooksModel(QAbstractTableModel):  # {{{
 
     about_to_be_sorted   = pyqtSignal(object, name='aboutToBeSorted')
@@ -168,6 +175,14 @@ class BooksModel(QAbstractTableModel):  # {{{
 
     def __init__(self, parent=None, buffer=40):
         QAbstractTableModel.__init__(self, parent)
+        base_font = parent.font() if parent else QApplication.instance().font()
+        self.bold_font = QFont(base_font)
+        self.bold_font.setBold(True)
+        self.italic_font = QFont(base_font)
+        self.italic_font.setItalic(True)
+        self.bi_font = QFont(self.bold_font)
+        self.bi_font.setItalic(True)
+        self.styled_columns = {}
         self.orig_headers = {
                         'title'     : _("Title"),
                         'ondevice'   : _("On Device"),
@@ -251,6 +266,21 @@ class BooksModel(QAbstractTableModel):  # {{{
                 self.dataChanged.emit(self.index(row, col), self.index(row,
                     col))
 
+    def change_column_font(self, colname, font_type):
+        if colname in self.column_map and font_type in ('normal', 'bold', 'italic', 'bi'):
+            db = self.db.new_api
+            old = db.pref('styled_columns', {})
+            old.pop(colname, None)
+            self.styled_columns.pop(colname, None)
+            if font_type != 'normal':
+                self.styled_columns[colname] = getattr(self, '{}_font'.format(font_type))
+                old[colname] = font_type
+            self.db.new_api.set_pref('styled_columns', old)
+            col = self.column_map.index(colname)
+            for row in xrange(self.rowCount(QModelIndex())):
+                self.dataChanged.emit(self.index(row, col), self.index(row,
+                    col))
+
     def is_custom_column(self, cc_label):
         try:
             return cc_label in self.custom_columns
@@ -273,6 +303,10 @@ class BooksModel(QAbstractTableModel):  # {{{
 
     def set_database(self, db):
         self.ids_to_highlight = []
+
+        if db:
+            style_map = {'bold': self.bold_font, 'bi': self.bi_font, 'italic': self.italic_font}
+            self.styled_columns = {k: style_map.get(v, None) for k, v in db.new_api.pref('styled_columns', {}).iteritems()}
         self.alignment_map = {}
         self.ids_to_highlight_set = set()
         self.current_highlighted_idx = None
@@ -280,6 +314,7 @@ class BooksModel(QAbstractTableModel):  # {{{
         self.custom_columns = self.db.field_metadata.custom_field_metadata()
         self.column_map = list(self.orig_headers.keys()) + \
                           list(self.custom_columns)
+
         def col_idx(name):
             if name == 'ondevice':
                 return -1
@@ -326,9 +361,10 @@ class BooksModel(QAbstractTableModel):  # {{{
                 self.new_bookdisplay_data.emit(self.get_book_display_info(current_row))
 
     def close(self):
-        self.db.close()
-        self.db = None
-        self.beginResetModel(), self.endResetModel()
+        if self.db is not None:
+            self.db.close()
+            self.db = None
+            self.beginResetModel(), self.endResetModel()
 
     def add_books(self, paths, formats, metadata, add_duplicates=False,
             return_ids=False):
@@ -376,6 +412,9 @@ class BooksModel(QAbstractTableModel):  # {{{
 
     def delete_books_by_id(self, ids, permanent=False):
         self.db.new_api.remove_books(ids, permanent=permanent)
+        self.ids_deleted(ids)
+
+    def ids_deleted(self, ids):
         self.db.data.books_deleted(tuple(ids))
         self.db.notify('delete', list(ids))
         self.books_deleted()
@@ -632,6 +671,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                     except:
                         traceback.print_exc()
                 pt.close()
+
                 def to_uni(x):
                     if isbytestring(x):
                         x = x.decode(filesystem_encoding)
@@ -726,6 +766,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                 bt = self.db.new_api.pref('bools_are_tristate')
                 bn = self.bool_no_icon
                 by = self.bool_yes_icon
+
                 def func(idx):
                     val = force_to_bool(fffunc(field_obj, idfunc(idx)))
                     if val is None:
@@ -733,6 +774,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                     return by if val else bn
             elif field == 'size':
                 sz_mult = 1.0/(1024**2)
+
                 def func(idx):
                     val = fffunc(field_obj, idfunc(idx), default_value=0) or 0
                     if val is 0:
@@ -745,6 +787,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             elif field == 'ondevice' and decorator:
                 by = self.bool_yes_icon
                 bb = self.bool_blank_icon
+
                 def func(idx):
                     return by if fffunc(field_obj, idfunc(idx)) else bb
             elif dt in {'text', 'comments', 'composite', 'enumeration'}:
@@ -754,6 +797,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                     if field_obj.is_composite:
                         if do_sort:
                             sv = m['is_multiple']['cache_to_list']
+
                             def func(idx):
                                 val = fffunc(field_obj, idfunc(idx), default_value='') or ''
                                 return (jv.join(sorted((x.strip() for x in val.split(sv)), key=sort_key)))
@@ -780,10 +824,12 @@ class BooksModel(QAbstractTableModel):  # {{{
                     return (QDateTime(as_local_time(fffunc(field_obj, idfunc(idx), default_value=UNDEFINED_DATE))))
             elif dt == 'rating':
                 rating_fields[field] = m['display'].get('allow_half_stars', False)
+
                 def func(idx):
                     return int(fffunc(field_obj, idfunc(idx), default_value=0))
             elif dt == 'series':
                 sidx_field = self.db.new_api.fields[field + '_index']
+
                 def func(idx):
                     book_id = idfunc(idx)
                     series = fffunc(field_obj, book_id, default_value=False)
@@ -792,6 +838,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                     return None
             elif dt in {'int', 'float'}:
                 fmt = m['display'].get('number_format', None)
+
                 def func(idx):
                     val = fffunc(field_obj, idfunc(idx))
                     if val is None:
@@ -822,6 +869,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                 self.dc_decorator[col] = renderer(col, 'bool')
 
         tc = self.dc.copy()
+
         def stars_tooltip(func, allow_half=True):
             def f(idx):
                 ans = val = int(func(idx))
@@ -913,11 +961,9 @@ class BooksModel(QAbstractTableModel):  # {{{
             self.column_color.mi = None
             return None
         elif role == Qt.DecorationRole:
+            default_icon = None
             if self.column_to_dc_decorator_map[col] is not None:
-                ccicon = self.column_to_dc_decorator_map[index.column()](index.row())
-                if ccicon is not None:
-                    return ccicon
-
+                default_icon = self.column_to_dc_decorator_map[index.column()](index.row())
             rules = self.db.prefs['column_icon_rules']
             if rules:
                 key = self.column_map[col]
@@ -939,15 +985,19 @@ class BooksModel(QAbstractTableModel):  # {{{
                                   self.icon_template_cache)
                     if ccicon is not None:
                         return ccicon
-                    if need_icon_with_text:
+                    if need_icon_with_text and default_icon is None:
                         self.icon_cache[id_][cache_index] = self.bool_blank_icon
                         return self.bool_blank_icon
                     self.icon_cache[id_][cache_index] = None
+            return default_icon
         elif role == Qt.TextAlignmentRole:
             cname = self.column_map[index.column()]
             ans = Qt.AlignVCenter | ALIGNMENT_MAP[self.alignment_map.get(cname,
                 'left')]
             return (ans)
+        elif role == Qt.FontRole and self.styled_columns:
+            cname = self.column_map[index.column()]
+            return self.styled_columns.get(cname)
         # elif role == Qt.ToolTipRole and index.isValid():
         #    if self.column_map[index.column()] in self.editable_cols:
         #        return (_("Double click to <b>edit</b> me<br><br>"))
@@ -1073,11 +1123,11 @@ class BooksModel(QAbstractTableModel):  # {{{
                 import traceback
                 if getattr(err, 'errno', None) == errno.EACCES:  # Permission denied
                     fname = getattr(err, 'filename', None)
-                    p = 'Locked file: %s\n\n'%fname if fname else ''
+                    p = 'Locked file: %s\n\n'%force_unicode(fname if fname else '')
                     error_dialog(get_gui(), _('Permission denied'),
                             _('Could not change the on disk location of this'
                                 ' book. Is it open in another program?'),
-                            det_msg=p+traceback.format_exc(), show=True)
+                            det_msg=p+force_unicode(traceback.format_exc()), show=True)
                     return False
                 error_dialog(get_gui(), _('Failed to set data'),
                         _('Could not set data, click Show Details to see why.'),
@@ -1140,6 +1190,8 @@ class BooksModel(QAbstractTableModel):  # {{{
                 val = val.split(',')
                 self.db.set_languages(id, val)
             else:
+                if column == 'authors' and val:
+                    val = authors_to_string(string_to_authors(val))
                 books_to_refresh |= self.db.set(row, column, val,
                                                 allow_case_change=True)
             self.refresh_ids(list(books_to_refresh), row)
@@ -1148,6 +1200,7 @@ class BooksModel(QAbstractTableModel):  # {{{
 
 # }}}
 
+
 class OnDeviceSearch(SearchQueryParser):  # {{{
 
     USABLE_LOCATIONS = [
@@ -1239,6 +1292,7 @@ class OnDeviceSearch(SearchQueryParser):  # {{{
 
 # }}}
 
+
 class DeviceDBSortKeyGen(object):  # {{{
 
     def __init__(self, attr, keyfunc, db):
@@ -1254,6 +1308,7 @@ class DeviceDBSortKeyGen(object):  # {{{
         return ans
 # }}}
 
+
 class DeviceBooksModel(BooksModel):  # {{{
 
     booklist_dirtied = pyqtSignal()
@@ -1406,6 +1461,7 @@ class DeviceBooksModel(BooksModel):  # {{{
     def sort(self, col, order, reset=True):
         descending = order != Qt.AscendingOrder
         cname = self.column_map[col]
+
         def author_key(x):
             try:
                 ax = self.db[x].author_sort
@@ -1690,6 +1746,3 @@ class DeviceBooksModel(BooksModel):  # {{{
             self.editable = []
 
 # }}}
-
-
-
